@@ -3,8 +3,6 @@
 sudo apt-get install -y \
     curl \
     gawk \
-    libcapture-tiny-perl \
-    libcarp-assert-perl \
     libclone-perl \
     libdata-compare-perl \
     libdate-calc-perl \
@@ -16,7 +14,6 @@ sudo apt-get install -y \
     libhtml-parser-perl \
     libjson-perl \
     liblist-moreutils-perl \
-    libchemistry-openbabel-perl \
     libparse-yapp-perl \
     libtext-unidecode-perl \
     libwww-curl-perl \
