@@ -1,8 +1,8 @@
 #------------------------------------------------------------------------------
 #$Author: antanas $
-#$Date: 2019-11-15 17:58:23 +0200 (Fri, 15 Nov 2019) $
-#$Revision: 7411 $
-#$URL: svn://www.crystallography.net/cod-tools/tags/v2.10/src/lib/perl5/COD/CIF/Data/Check.pm $
+#$Date: 2020-03-10 16:36:52 +0200 (Tue, 10 Mar 2020) $
+#$Revision: 7772 $
+#$URL: svn+ssh://www.crystallography.net/home/coder/svn-repositories/cod-tools/tags/v3.1.0/src/lib/perl5/COD/CIF/Data/Check.pm $
 #------------------------------------------------------------------------------
 #*
 #* Perform various checks of the CIF data.
@@ -190,9 +190,9 @@ sub check_formula_sum_syntax
     if ( $formula !~ /^\s*(?:$formula_comp\s+)*(?:$formula_comp)\s*$/ ) {
         push @messages,
              "WARNING, chemical formula '$formula' could not be parsed -- "
-           . 'a chemical formula should consist of a space-seprated '
+           . 'a chemical formula should consist of space-separated '
            . 'chemical element symbols with optional numeric quantities '
-           . "(e.g. 'C2 H6 O')";
+           . '(e.g. \'C2 H6 O\')';
     }
 
     return \@messages;
